SystemOrganization addCategory: #'Seaside-Environment'!

!WADispatcher class methodsFor: '*seaside-environment' stamp: 'jf 9/24/2008 10:38'!
resetAll
	self seasideDeprecatedApi: 'WADispatcher class>>resetAll'
		details: 'Use WAAdmin>>reloadApplications.'.
	WAAdmin reloadApplications! !

WAErrorHandler subclass: #WAHtmlErrorHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Environment'!

!WAHtmlErrorHandler commentStamp: 'jf 9/17/2008 20:52' prior: 0!
This error handler returns a simple HTML error message response.

It provides two methods to allow the easy creation of custom error responses:

#titleForException:  - can be used to override the text displayed in the title
#renderContentForException:  - can be used to fully customize the HTML output!

!WAHtmlErrorHandler methodsFor: 'actions' stamp: 'jf 9/14/2008 20:30'!
handleDefault: anException
	| response content |
	response := WAResponse new.
	response internalError.
	content := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :root | root title: (self titleForException: anException) ];
		render: [ :html | self renderContentForException: anException on: html ].
	response nextPutAll: content.
	^ response! !

!WAHtmlErrorHandler methodsFor: 'actions' stamp: 'jf 9/14/2008 20:32'!
renderContentForException: anException on: html
	html heading level1; with: anException description.
	html paragraph: 'Your request could not be completed. An error occurred.'! !

!WAHtmlErrorHandler methodsFor: 'content' stamp: 'jf 9/14/2008 20:28'!
titleForException: anException
	^ anException description! !

!WAPackage class methodsFor: '*Seaside-Environment' stamp: 'lr 10/23/2008 21:13'!
seasideEnvironment
	^ self new
		name: 'Seaside-Environment';
		description: 'Administrative tools, useful to have for productive and development environments.';
		addDependency: 'Seaside-Component';
		addDependency: 'Seaside-RenderLoop';
		addDependency: 'Seaside-Session';
		addDependency: 'Seaside-Canvas';
		yourself! !

WAFileLibrary subclass: #WAStandardFiles
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Environment'!

!WAStandardFiles commentStamp: 'jf 10/6/2008 21:39' prior: 0!
A collection of standard scripts, styles, and images.

#windowCss styles the HTML created by WAWindowDecoration.
#shortcutJs creates the JavaScript used by WATagBrush>>addShortcut:.
#miscJs creates JavaScript used by:
	- WACheckboxTag>>submitFormNamed:
	- WAFormInputTag>>setFocus
	- WAAbstractTextAreaTag>>setSelectionFrom:to:
	- (the rest of the JS functions appear to be uncalled in the Seaside codebase)
#kalseyTabCss styles the HTML created by WANavigation
#externalAnchorJs sets a target of _blank for any anchor that has a "rel" attribute
	of "external". However, the JS function does not appear to be called anywhere
	in Seaside.!

!WAStandardFiles methodsFor: 'accessing-scripts' stamp: 'pmm 2/13/2007 19:16'!
externalAnchorsJs
	^ '// See http://www.sitepoint.com/article/standards-compliant-world
function externalLinks() {
	if (!!document.getElementsByTagName) return;
	var anchors = document.getElementsByTagName("a");
	for (var i=0; i<anchors.length; i++) {
	var anchor = anchors[i];
	if (anchor.getAttribute("href") && anchor.getAttribute("rel") == "external")
		anchor.target = "_blank";
	}
}'! !

!WAStandardFiles methodsFor: 'accessing-styles' stamp: 'lr 10/28/2007 14:42'!
kalseyTabsCss
	"this is from http://kalsey.com/tools/csstabs/ - would also be nice to do http://www.alistapart.com/articles/slidingdoors/"
	^ '
.kalsey .navigation-options {
	border-bottom : 1px solid #ccc;
	margin : 0;
	padding-bottom : 19px;
	padding-left : 10px;
}

.kalsey .navigation-options ul, .kalsey .navigation-options li	{
	display : inline;
	list-style-type : none;
	margin : 0;
	padding : 0;
}


.kalsey .navigation-options a:link, .kalsey .navigation-options a:visited	{
	background : #E8EBF0;
	border : 1px solid #ccc;
	color : #666;
	float : left;
	font-size : small;
	font-weight : normal;
	line-height : 14px;
	margin-right : 8px;
	padding : 2px 10px 2px 10px;
	text-decoration : none;
}

.kalsey .navigation-options a:link.active, .kalsey .navigation-options a:visited.active	{
	background : #fff;
	border-bottom : 1px solid #fff;
	color : #000;
}

.kalsey .navigation-options ul a:hover	{
	color : #f00;
}

.kalsey .navigation-options .option-selected a {
	background : #fff;
	border-bottom : 1px solid #fff;
	color : #000;
}

.kalsey .navigation-content {
	background : #fff;
	border : 1px solid #ccc;
	border-top : none;
	clear : both;
	margin : 0px;
	padding : 15px;
	line-height: 1.1;  /* IE6 CSS workaround - http://www.dracos.co.uk/web/css/ie6floatbug/ */
}
'! !

!WAStandardFiles methodsFor: 'accessing-scripts' stamp: 'lr 10/28/2007 14:42'!
miscJs
	^ '
	function swapDisplay(a, b) {
		var tmp = document.getElementById(a).style.display;
		document.getElementById(a).style.display = document.getElementById(b).style.display;
		document.getElementById(b).style.display = tmp;
	}

	function submitFormTriggeringCallback(formName, callbackKey, value) {
		if (value)
			{
				var ele = document.createElement("input");
				ele.type = "hidden";
				ele.name = callbackKey;
				ele.value = value;
				document.forms[formName].appendChild(ele)
			}
		submitForm(formName)
	}

	function submitForm(formName) {
		document.forms[formName].submit();
	}

	function chooseOther(select, hiddenId, p) {
		value = prompt(p);
		document.getElementById(hiddenId).value = value;
		select.options[select.options.length-1].text = value;
	}

	function enableChoice(enableID, disableID) {
		document.getElementById(enableID).disabled = false;
		document.getElementById(disableID).disabled = true;
	}

	function setFocus(elementId) {
		document.getElementById(elementId).focus();
	}

	function setSelection(elementId, start, stop) {
		var input = document.getElementById(elementId);
		input.focus();
		/* When stop parameter not supplied, all field is selected */
		if (!!start) start = 1;
		if (!!stop) stop = input.value.length + 1;
		if (typeof document.selection !!= "undefined") {
	    		/* Place selection in MS-IE */
	    		var range = document.selection.createRange();
	     	range.moveStart("character", start - 1);
	     	range.moveEnd("character", stop - start);
	 		range.select();
	  	} else if (typeof input.selectionStart !!= "undefined") {
		  	/* Place selection in Gecko browsers */
	    		input.selectionStart = start - 1;
	    		input.selectionEnd = stop - 1;
	  	} else {
	  		/* Other browsers, please feel free to implement */
			alert("unknown browser");
		}
	}

'! !

!WAStandardFiles methodsFor: 'accessing' stamp: 'MAD 8/29/2008 16:43'!
selectorsToInclude
	^#(
		windowCss

		kalseyTabsCss
		externalAnchorsJs
		miscJs
		shortcutsJs
	)! !

!WAStandardFiles methodsFor: 'accessing-scripts' stamp: 'lr 10/28/2007 14:42'!
shortcutsJs
	^ '

	var shortcutKeys = [];
	var shortcutElements = [];

	function resetShortcuts() {
		shortcutKeys = [];
		shortcutElements = [];
	}

	function addShortcut(shortcut, elementID) {
		var elem = document.getElementById(elementID);
		if ((elem.tagName == "INPUT" & (elem.type == "submit" || elem.type == "checkbox" || elem.type == "radio")) || elem.tagName == "A")
			{
			shortcutKeys[shortcutKeys.length] = shortcut;
			shortcutElements[shortcutElements.length] = elem;
			}
		else alert("Attempt to assign a shortcut (" + shortcut + ") to something that is not clickable");
	}

	function onKeyDown(event) {
		var keyname = "";
		var element;
		var nav;
		if (navigator.userAgent.indexOf("Safari") > 0)
			nav = "Safari";
		else if (navigator.product == "Gecko")
			nav = "Gecko";
		else
			nav = "IE";
		// cope with MS-IE
		if (event == null) event = window.event;
		if (event) {
			if (event.ctrlKey) keyname = keyname + "Ctrl-";
			if (event.altKey) keyname = keyname + "Alt-";
			if (event.metaKey) keyname = keyname + "Meta-";
			// cope with Netscape
			var keyCode = event.which;
			if (keyCode == null) keyCode = event.keyCode;
			if (keyCode == null) keyCode = event.charCode;
			if (keyCode !!= 0) {
				var character = String.fromCharCode(keyCode);
				character = character.toLowerCase();
				if (event.shiftKey) character = character.toUpperCase();
				// cope with special keys, designated with some logical names
				if (keyCode == 27) character = "Esc";
				if (keyCode == 8) character = "Backspace";
				if (keyCode == 9) character = "Tab";
				if (keyCode == 13) character = "Return";
				if (keyCode == 37) character = "Left";
				if (keyCode == 38) character = "Up";
				if (keyCode == 39) character = "Right";
				if (keyCode == 40) character = "Down";
				// cope with func-keys (some keys not intercepted for MS-IE)
				if ((keyCode >= 112) & (keyCode <= 123)) character = "F" + (keyCode - 112 + 1);
				for ( var x = 1 ; x <= shortcutKeys.length ; x++ ) {
					if (shortcutKeys[x-1] == keyname+character) {
						// alert("key=" + keyCode + " " + keyname + character);
						element = shortcutElements[x-1];
						if (element.tagName == "A" & nav !!= "IE") {
							if (element.onclick)
								element.onclick.call();
							else
								/* This does not work for anchor in IE */
								location.href = element.href;
						} else  {
							element.click();
						}
					}
				}
				return false;
			}
		}
	}

'! !

!WAStandardFiles methodsFor: 'processing' stamp: 'lr 10/28/2007 14:42'!
updateRoot: aHtmlRoot
	super updateRoot: aHtmlRoot.
	aHtmlRoot bodyAttributes
		at: 'onkeydown'
		put: 'onKeyDown(event)'! !

!WAStandardFiles methodsFor: 'accessing-styles' stamp: 'MAD 8/29/2008 19:38'!
windowCss
	^ '.window {
	border: 1px solid #aaa;
}
.window .window-titlebar {
	color: #444;
	padding: 5px;
	overflow: hidden;
	background-color: #eee;
	border-bottom: 1px solid #aaa;
	font: 10pt Verdana, Arial, Helvetica, sans-serif;
}
.window .window-titlebar .window-title {
	float: left;
	font-weight: bold;
	padding-left: 20px;
}
.window .window-titlebar .window-close {
	float: right;
}
.window .window-titlebar .window-close a {
	color: #444;
	text-decoration: none;
}
.window .window-titlebar .window-close a:hover {
	color: #111;
}
.window .window-content {
	clear: both;
	padding: 2px;
}'! !

WAObject subclass: #WAAdmin
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Environment'!

!WAAdmin class methodsFor: 'private' stamp: 'lr 10/21/2008 15:40'!
applicationDefaults
	| config |
	config := WASharedConfiguration named: 'Application Defaults'.
	config parents isEmpty ifFalse: [ ^ config ].
	config
		addParent: WAApplicationConfiguration instance;
		addParent: WARenderLoopConfiguration instance;
		at: #exceptionHandler put: WAHtmlErrorHandler;
		at: #libraries addAll: (Array with: WAStandardFiles);
		at: #responseFactory put: WAHtmlResponseFactory.
	^ config! !

!WAAdmin class methodsFor: 'development' stamp: 'jf 9/17/2008 20:58'!
clearConfigurationCaches
	WASystemConfiguration clearAllDescriptions! !

!WAAdmin class methodsFor: 'development' stamp: 'jf 9/14/2008 10:23'!
clearSessions
	WARegistry clearAllHandlers! !

!WAAdmin class methodsFor: 'initialization' stamp: 'lr 10/21/2008 15:35'!
initialize
	self reloadApplications! !

!WAAdmin class methodsFor: 'registration' stamp: 'jf 10/6/2008 20:56'!
register: aComponent at: aString
	"Use this to programatically register a component as an application."
	"WARenderLoopConfiguration is added by default because most users will use WASession and the Render Loop."
	| application |
	application := WAApplication register: aString.
	application configuration
		removeParent: WASessionConfiguration instance; "make the chain slightly cleaner"
		addParent: WARenderLoopConfiguration instance;
		addParent: self applicationDefaults.
	application preferenceAt: #rootClass put: aComponent.
	^ application! !

!WAAdmin class methodsFor: 'registration' stamp: 'MAD 9/13/2008 17:10'!
register: aComponent at: aString user: user password: password
	"Like #registerApplicationAt: root: but additionally adds password protection."
	| application |
	application := self register: aComponent at: aString.
	application configuration addParent: WAAuthConfiguration instance.
	application 
		preferenceAt: #login
		put: user.
	application 
		preferenceAt: #passwordHash
		put: (SeasidePlatformSupport secureHashFor: password).
	application
		addFilter: WAAuthenticationFilter new.
	^ application! !

!WAAdmin class methodsFor: 'registration' stamp: 'lr 10/21/2008 15:32'!
registerDefaultComponents
	WAComponent allSubclasses do: [ :each |
		(each class includesSelector: #initialize)
			ifTrue: [ each initialize ] ]! !

!WAAdmin class methodsFor: 'registration' stamp: 'lr 10/21/2008 15:35'!
registerDefaultRequestHandlers
	WAFileHandler default: (WAFileHandler named: 'files').
	WADispatcher default register: WAFileHandler default! !

!WAAdmin class methodsFor: 'development' stamp: 'lr 10/21/2008 15:30'!
reloadApplications
	WADispatcher resetDefault.
	WASystemConfiguration clearAllDescriptions.
	self registerDefaultRequestHandlers.
	self registerDefaultComponents! !

!WAComponent class methodsFor: '*Seaside-Environment' stamp: 'jf 9/24/2008 10:41'!
registerAsApplication: aString 
	self seasideDeprecatedApi: 'WAComponent class>>registerAsApplication:'
		details: 'Use WAAdmin>>register:at:'.
	^ WAAdmin 	register: self at: aString! !

WAResponseFactory subclass: #WAHtmlResponseFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Environment'!

!WAHtmlResponseFactory class methodsFor: 'as yet unclassified' stamp: 'jf 9/14/2008 00:36'!
forbiddenForContext: aRequestContext
	| response content |
	response := WAResponse new.
	response forbidden.
	content := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :root | root title: 'Forbidden' ];
		render: [ :html |
			html heading level1; with: [
				html text: 'Error: you are forbidden to access "'.
				html text: aRequestContext request url.
				html text: '"' ] ].
	response nextPutAll: content.
	^ response! !

!WAHtmlResponseFactory class methodsFor: 'as yet unclassified' stamp: 'jf 9/14/2008 00:35'!
notFoundForContext: aRequestContext 
	| response content |
	response := WAResponse new.
	response notFound.
	content := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :root | root title: 'Not Found' ];
		render: [ :html |
			html heading level1; with: [
				html text: 'Error: "'.
				html text: aRequestContext request url.
				html text: '" not found' ] ].
	response nextPutAll: content.
	^ response! !

!WAHtmlResponseFactory class methodsFor: 'as yet unclassified' stamp: 'pmm 9/14/2008 13:56'!
pageExpiredForContext: aRequestContext
	| content |
	content := WARenderCanvas builder
		fullDocument: true;
		rootBlock: [ :root | root title: 'Page Expired' ];
		render: [ :html | 
			html heading: 'Page Expired'.
			html paragraph: 'This page has expired. Please use the back-button to navigate to the last valid page.' ].
	^WAResponse new
		nextPutAll: content;
		doNotCache;
		yourself! !

!WAHtmlResponseFactory class methodsFor: 'as yet unclassified' stamp: 'jf 9/14/2008 00:36'!
refreshAfter: aNumber location: aUrlOrString message: aString
	| response document |
	response := WAResponse new.
	response refreshAfter: aNumber location: aUrlOrString.
	
	document := WARenderCanvas builder
		fullDocument: true;
		render: [ :html |
			html heading level1; with: aString.
			html paragraph: [
				html text: 'You are being redirected to '.
				html anchor
					url: aUrlOrString;
					with: aUrlOrString seasideString ] ].
	
	response nextPutAll: document.
		
	^ response! !
WAAdmin initialize!
